@@ -6,22 +6,109 @@ from __future__ import print_function, division
 # - Optional FITS input/output
 # - Output convolved fluxes
 
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
 import numpy as np
 from astropy import units as u
 
 from . import timer
 
 from .models import Models
-from .fit_info import FitInfo
 from .source import Source
 from .utils import io
-from .utils.validator import validate_scalar, validate_array
+from .utils.validator import validate_array
 from . import six
+from .fit_info import FitInfoFile
+
+
+class Fitter(object):
+    """
+    A fitter class that can be used to fit sources.
+
+    This class is initialized using a particular set of models, and with
+    specific fit parameters. It can then be used to fit data given by Source
+    instances, and returns a FitInfo instance. Once initialized, the fit
+    parameters cannot be changed, because changing most of them would require
+    re-reading the models from disk.
+
+    Parameters
+    ----------
+    filter_names : tuple or list
+        List of filter names (given as individual strings) for which the data
+        is defined. The filter names should be the name of the files in the
+        ``convolved`` directory for the models, without the extensions. This is
+        typically ``2J``, ``I1``, ``M1``, etc. You can also specify the
+        wavelength as a :class:`~astropy.units.quantity.Quantity` instance
+        instead of a filter name, and this will indicate that the SED fluxes
+        closest to the requested wavelength should be used in the fitting.
+    apertures : :class:`~astropy.units.quantity.Quantity` array instance
+        The aperture radii that the data is specified in (as an angle). The
+        fluxes may not be measured from aperture photometry, but this is meant
+        to give an indication of the sizescale of the emission, and can be used
+        to reject models that would have been clearly resolved at the distance
+        specified.
+    models_dir : str
+        Name of the directory containing the models to use.
+    extinction_law : :class:`~sedfitter.extinction.Extinction` instance
+        The extinction law to use.
+    av_range : tuple
+        Minimum and maximum Av to allow in the fitting.
+    distance_range : :class:`~astropy.units.quantity.Quantity` array instance
+        Minimum and maximum distance to allow in the fitting in units of length.
+    remove_resolved : bool, optional
+        If set, then models larger than the aperture are removed. See
+        Robitaille et al. (2007) for a discussion of this criterion.
+    """
+
+    def __init__(self, filter_names, apertures, model_dir,
+                 extinction_law=None, av_range=None, distance_range=None,
+                 remove_resolved=False):
+
+        validate_array('apertures', apertures, domain='positive', ndim=1, physical_type='angle')
+        validate_array('distance_range', distance_range, domain='positive', ndim=1, shape=(2,), physical_type='length')
+
+        if len(apertures) != len(filter_names):
+            raise ValueError("length of apertures list should match length of filter names list")
+
+        # Construct filters dictionary
+        self.filters = []
+        for i in range(len(apertures)):
+            filt = {'aperture_arcsec': apertures[i].to(u.arcsec).value}
+            if isinstance(filter_names[i], six.string_types):
+                filt['name'] = filter_names[i]
+            elif isinstance(filter_names[i], u.Quantity):
+                filt['wav'] = filter_names[i]
+            else:
+                raise ValueError("filter should be a string or a Quantity")
+
+            self.filters.append(filt)
+
+        # Read in models
+        self.models = Models.read(model_dir, self.filters, distance_range=distance_range, remove_resolved=remove_resolved)
+
+        # Add wavelength to filters
+        for i, f in enumerate(self.filters):
+            if 'wav' not in f:
+                f['wav'] = self.models.wavelengths[i]
+
+        # Set Av law
+        self.av_law = extinction_law.get_av(self.models.wavelengths)
+
+        # Set scale model - make this a scalar
+        self.sc_law = -2. * np.ones(self.av_law.shape)
+
+        self.model_dir = model_dir
+        self.av_range = av_range
+        self.extinction_law = extinction_law
+
+    def fit(self, source):
+
+        info = self.models.fit(source, self.av_law, self.sc_law,
+                               self.av_range[0], self.av_range[1])
+
+        info.meta.model_dir = self.model_dir
+        info.meta.filters = self.filters
+        info.meta.extinction_law = self.extinction_law
+
+        return info
 
 
 def fit(data, filter_names, apertures, model_dir, output, n_data_min=3,
@@ -40,7 +127,10 @@ def fit(data, filter_names, apertures, model_dir, output, n_data_min=3,
         List of filter names (given as individual strings) for which the data
         is defined. The filter names should be the name of the files in the
         ``convolved`` directory for the models, without the extensions. This is
-        typically ``2J``, ``I1``, ``M1``, etc.
+        typically ``2J``, ``I1``, ``M1``, etc. You can also specify the
+        wavelength as a :class:`~astropy.units.quantity.Quantity` instance
+        instead of a filter name, and this will indicate that the SED fluxes
+        closest to the requested wavelength should be used in the fitting.
     apertures : :class:`~astropy.units.quantity.Quantity` array instance
         The aperture radii that the data is specified in (as an angle). The
         fluxes may not be measured from aperture photometry, but this is meant
@@ -70,8 +160,10 @@ def fit(data, filter_names, apertures, model_dir, output, n_data_min=3,
         Robitaille et al. (2007) for a discussion of this criterion.
     """
 
-    validate_array('apertures', apertures, domain='positive', ndim=1, physical_type='angle')
-    validate_array('distance_range', distance_range, domain='positive', ndim=1, shape=(2,), physical_type='length')
+    fitter = Fitter(filter_names, apertures, model_dir,
+                    extinction_law=extinction_law, av_range=av_range,
+                    distance_range=distance_range,
+                    remove_resolved=remove_resolved)
 
     print(" ------------------------------------------------------------")
     print("  => Fitting parameters")
@@ -103,44 +195,18 @@ def fit(data, filter_names, apertures, model_dir, output, n_data_min=3,
     else:
         data_file = data
 
-    if len(apertures) != len(filter_names):
-        raise ValueError("length of apertures list should match length of filter names list")
-
-    # Construct filters dictionary
-    filters = []
-    for i in range(len(apertures)):
-        filters.append({'aperture_arcsec': apertures[i].to(u.arcsec).value, 'name': filter_names[i]})
-
-    # Read in models
-    models = Models.read(model_dir, filters, distance_range=distance_range, remove_resolved=remove_resolved)
-
-    # Add wavelength to filters
-    for i, f in enumerate(filters):
-        f['wav'] = models.wavelengths[i]
-
     print('')
     print('     Filter    Wavelength    Aperture (")   ')
     print('    ----------------------------------------')
-    for f in filters:
-        print('       %5s   %9.2f  %9.2f        ' % (f['name'], f['wav'].to(u.micron).value, f['aperture_arcsec']))
+    for f in fitter.filters:
+        print('       %5s   %9.2f  %9.2f        ' % (f.get('name', ''), f['wav'].to(u.micron).value, f['aperture_arcsec']))
     print('')
 
-    # Set Av law
-    av_law = extinction_law.get_av(models.wavelengths)
-
-    # Set scale model - make this a scalar
-    sc_law = -2. * np.ones(av_law.shape)
-
     # Cycle through sources
 
     io.delete_file(output)
 
-    fout = open(output, 'wb')
-    pickle.dump(model_dir, fout, 2)
-    pickle.dump(filters, fout, 2)
-    pickle.dump(extinction_law, fout, 2)
-
-    # NOTE _ CAN USE PROTOCOL 2 IN SOURCE FOR COORINDATES
+    fout = FitInfoFile(output, 'w')
 
     s = Source()
 
@@ -155,16 +221,14 @@ def fit(data, filter_names, apertures, model_dir, output, n_data_min=3,
 
         if s.n_data >= n_data_min:
 
-            info = FitInfo(source=s)
-            info.av, info.sc, info.chi2, info.model_name, model_fluxes = models.fit(s, av_law, sc_law, av_range[0], av_range[1])
+            info = fitter.fit(s)
 
-            if output_convolved:
-                info.model_fluxes = model_fluxes
+            if not output_convolved:
+                info.model_fluxes = None
 
-            info.sort()
             info.keep(output_format[0], output_format[1])
 
-            pickle.dump(info, fout, 2)
+            fout.write(info)
 
             t.display()
 
